#ifndef MYSOLVER_HH
#define MYSOLVER_HH

#include <dune/common/bitsetvector.hh>
#include <dune/common/parametertree.hh>

#include <dune/solvers/iterationsteps/multigridstep.hh>
#include <dune/solvers/iterationsteps/truncatedblockgsstep.hh>
#include <dune/solvers/norms/energynorm.hh>
#include <dune/solvers/solvers/loopsolver.hh>
#include <dune/tnnmg/iterationsteps/genericnonlineargs.hh>
#include <dune/solvers/transferoperators/compressedmultigridtransfer.hh>
#include <dune/fufem/assemblers/transferoperatorassembler.hh>
#include <dune/tnnmg/iterationsteps/tnnmgstep.hh>
#include <dune/tnnmg/problem-classes/convexproblem.hh>

#include "dune/tectonic/myblockproblem.hh"
#include "dune/tectonic/globalnonlinearity.hh"

template <int dim, class MatrixType, class VectorType, class GridType>
class MySolver {
private:
  using ConvexProblemType = ConvexProblem<
      Dune::GlobalNonlinearity<MatrixType, VectorType>, MatrixType>;
  using BlockProblemType = MyBlockProblem<ConvexProblemType>;
  using NonlinearSmootherType = GenericNonlinearGS<BlockProblemType>;
  using SolverType = TruncatedNonsmoothNewtonMultigrid<BlockProblemType,
                                                       NonlinearSmootherType>;

public:
  MySolver(Dune::ParameterTree const &parset, int refinements,
           double solver_tolerance, GridType const &grid,
           Dune::BitSetVector<dim> const &ignoreNodes);

  ~MySolver();

  SolverType *getSolver();

private:
  TruncatedBlockGSStep<MatrixType, VectorType> linearBaseSolverStep;
  EnergyNorm<MatrixType, VectorType> baseEnergyNorm;
  LoopSolver<VectorType> linearBaseSolver;
  TruncatedBlockGSStep<MatrixType, VectorType> linearPresmoother;
  TruncatedBlockGSStep<MatrixType, VectorType> linearPostsmoother;
  MultigridStep<MatrixType, VectorType> linearIterationStep;
  std::vector<CompressedMultigridTransfer<VectorType> *> transferOperators;
  NonlinearSmootherType nonlinearSmoother;
  SolverType *multigridStep;
};

#endif
