#include <cassert>

#include <dune/tnnmg/problem-classes/bisection.hh>

#include <dune/tectonic/mydirectionalconvexfunction.hh>

#include "compute_state_dieterich_common.hh"
#include "compute_state_dieterich_euler.hh"

double state_update_dieterich_euler_bisection(double tau, double VoL,
                                              double old_state) {
  DieterichNonlinearity::VectorType const start(0);
  DieterichNonlinearity::VectorType const direction(1);
  DieterichNonlinearity const phi(VoL);
  MyDirectionalConvexFunction<DieterichNonlinearity> const J(
      1.0 / tau, old_state / tau, phi, start, direction);

  int bisectionsteps = 0;
  Bisection const bisection(0.0, 1.0, 1e-12, true, 0);    // TODO
  return bisection.minimize(J, 0.0, 0.0, bisectionsteps); // TODO
}

double state_update_dieterich_euler(double tau, double VoL, double old_state) {
  double const ret =
      state_update_dieterich_euler_bisection(tau, VoL, old_state);
  /* We have

     ret - old_state + tau * VoL = tau*exp(-ret)

     or

     log((ret - old_state)/tau + VoL) = -ret
  */
  assert(std::abs(ret - old_state + tau * (VoL - std::exp(-ret))) < 1e-12 ||
         std::abs(std::log((ret - old_state) / tau + VoL) + ret) < 1e-12);
  return ret;
}
