#ifndef DUNE_TECTONIC_TIMESTEPPING_IMPLICITEULER_HH
#define DUNE_TECTONIC_TIMESTEPPING_IMPLICITEULER_HH

template <class VectorType, class MatrixType, class FunctionType, int dim>
class ImplicitEuler
    : public TimeSteppingScheme<VectorType, MatrixType, FunctionType, dim> {
public:
  ImplicitEuler(MatrixType const &_A, VectorType const &_u_initial,
                VectorType const &_v_initial,
                Dune::BitSetVector<dim> const &_dirichletNodes,
                FunctionType const &_dirichletFunction);

  void virtual nextTimeStep() override;
  void virtual setup(VectorType const &, double, double, VectorType &,
                     VectorType &, MatrixType &) override;
  void virtual postProcess(VectorType const &) override;
  void virtual extractDisplacement(VectorType &) const override;
  void virtual extractVelocity(VectorType &) const override;

private:
  MatrixType const &A;
  VectorType u;
  VectorType v;
  Dune::BitSetVector<dim> const &dirichletNodes;
  FunctionType const &dirichletFunction;

  VectorType u_o;
  VectorType v_o;

  double tau;

  bool postProcessCalled = false;
};
#endif
